export { default as Js } from './ext.js';
export { default as Jsx } from './ext.jsx';
export { default as Es6 } from './ext.es6';
export { default as Es } from './ext.es';
export { default as Mjs } from './ext.mjs';
export { default as Json } from './ext.json';
export { default as Other } from './ext.other';
